/***************************************************************************
                            ezcmd  -  description
                             -------------------
    begin                : Jun. 6, 2008
    copyright            : (C) 2008 by Allen
    email                : bon_ami_@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   Explicit Distribution Limitation                                      *
 *   This rule overrides others below.                                     *
 *   This program may not be modified or used by, or, if possible,         *
 *   redistributed to people described as below,                           *
 *   1.Japanese who hold hostility against Chinese.                        *
 *   2.or, those who discriminate against people based solely on race,     *
 *     gender or sexual orientation.                                       *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*
 *	source code of EZ Command functionalities
 */

#ifdef WIN32
#include "stdafx.h"
#include <windows.h>
#else	/* UNIX? */
extern "C" {
#include <unistd.h>
}
#endif
extern "C" {
#include <stdlib.h>
}
#include <iostream>
#include <sstream>
#include <utility>
#include <vector>
#include <bitset>
#include <algorithm>
#include <ezproject/tinyxml.h>
#include <ezproject/ezcommon.h>

namespace ezproject
{

/*	EZ Project EZ Command functionalities	*/
class ezcmd
{
public:
	static const size_t flagIgnore = 0;
	static const size_t flagPrompt = 1;
	static const size_t flagStrict = 2;
	static const size_t flagQuantity = 3;
	int run(char *argFileConfig, ezoi &infoDisp,
		std::bitset<flagQuantity> &flags);

private:
#ifdef WIN32
	static TCHAR *char2TCHAR_begin(const char *str);
	//str must be NULL terminated
	static void char2TCHAR_end(const char *strOriginal,
		TCHAR *strConverted);
#endif
	static const std::string strDoc;
	static const std::string strCmd;
	static const std::string strChd;
	static const std::string strSet;
	static const std::string strGoto;
	static const std::string strName;
	static const std::string strPrompt;
	static const std::string strStrict;
	static const std::string strLabel;
	static const std::string strInput;
	enum eflow	//flow control
	{
		FLOW_STOP,
		FLOW_SKIP,
		FLOW_GOON
	};
	typedef int (*runCmdFunc)(ezcfg &cfg, ezoi &infoDisp);
	static void skipKeyParams(ezcfg &cfg, char **name, char **value);
	//skip keyword parameters
	static std::string displayCaption(ezoi &infoDisp, std::string &userCaption,
		const char *formalCaption, const char *formalEnding);
	//construct display string from user's configuration or a default setting
	static int runCmd(ezcfg &cfg, ezoi &infoDisp); //execute a command
	static int runChd(ezcfg &cfg, ezoi &infoDisp); //change directory
	static int runSet(ezcfg &cfg, ezoi &infoDisp);
	//set environment variables
	static int runSetExec(ezoi &infoDisp, char *name,
		char *value, bool verbose);
	static int runGoto(ezcfg &cfg, ezoi &infoDisp); //detailed flow control
	static std::string replaceText(ezcfg &cfg, ezoi &infoDisp);
	//replace node text according to input parameters
	eflow chkPermission(ezcfg &cfg, ezoi &infoDisp,
		std::bitset<flagQuantity> &flags);
	bool chkAbortion(int result, ezcfg &cfg,
		std::bitset<flagQuantity> &flags);
	int runExec(ezcfg &cfg, std::vector<std::pair<std::string, runCmdFunc> >
		&criteria, int requiredLevel,
		ezoi &infoDisp, std::bitset<flagQuantity> &flags);
	//process different elements in configuration
};

/*	keywords	*/
/*	nodes	*/
const std::string ezcmd::strDoc = "Document";
const std::string ezcmd::strCmd = "cmd";
const std::string ezcmd::strChd = "chdir";
const std::string ezcmd::strSet = "set";
const std::string ezcmd::strGoto = "goto";
/*	params	*/
const std::string ezcmd::strName = "name";
const std::string ezcmd::strPrompt = "prompt";
const std::string ezcmd::strStrict = "strict";
const std::string ezcmd::strLabel = "label";
const std::string ezcmd::strInput = "input";

#ifdef WIN32
TCHAR *ezcmd::char2TCHAR_begin(const char *str)
{
	int length;
	TCHAR *result = NULL;

	if (NULL == str)
		return NULL;

	length = MultiByteToWideChar(CP_ACP, 0, str, -1, result, 0);
	if (length <= 0)
		return NULL;
	result = new TCHAR[length];
	result[0] = '\0';	//necessary?
	length = MultiByteToWideChar(CP_ACP, 0, str, -1, result, length);
	if (length <= 0)
		delete[] result;
	return result;
}

void ezcmd::char2TCHAR_end(const char *strOriginal, TCHAR *strConverted)
{
	//if ((void *)strConverted != (void *)strOriginal)
	//	delete[] strConverted;
}
#endif	/* WIN32 */

void ezcmd::skipKeyParams(ezcfg &cfg, char **name, char **value)
{
	if (!name || (NULL == *name) || !strlen(*name))
		return;

	while ((strName == *name) || (strPrompt == *name)
		|| (strStrict == *name) || (strLabel == *name))
	{
		free(*name);
		*name = NULL;
		cfg.getAttribute(name, value);
		if (NULL == *name)
			break;
	}
}

std::string ezcmd::displayCaption(ezoi &infoDisp, std::string &userCaption,
	const char *formalCaption, const char *formalEnding)
{
	std::ostringstream info;

	if (userCaption.empty())
	{
		if (formalCaption)
			info << formalCaption;
		if (formalEnding)
			info << formalEnding;

		return info.str();
	}
	else
		return userCaption;
}
int ezcmd::runCmd(ezcfg &cfg, ezoi &infoDisp)
{
	int result = -1;
	std::string cmd = replaceText(cfg, infoDisp);

	if (!cmd.empty())
	{
		std::string attributeValue, caption;
		char *nm = strdup(strName.c_str()), *vl = NULL;

		cfg.getAttribute(&nm, &vl, true);
		if (vl)
		{
			attributeValue = vl;
			free(vl);
		}
		free(nm);
		caption = displayCaption(infoDisp,
			attributeValue, "executing: ", cmd.c_str());
		infoDisp.dsp(ezoi::OILVL_INFO, caption.c_str());

		result = system(cmd.c_str());
		if (result < 0)
		{
			std::ostringstream info;

			info << "failure (" << result
				<< "): " << caption;
			infoDisp.dsp(ezoi::OILVL_WARN, info.str().c_str());
		}
	}
	else
	{
		infoDisp.dsp((ezoi::OILVL_WARN),
			"no command found in an element");
		result = 1;
	}
	return result;
}


int ezcmd::runChd(ezcfg &cfg, ezoi &infoDisp)
{
	int result = -1;
	std::string cmd = replaceText(cfg, infoDisp);

	if (!cmd.empty())
	{
		std::string attributeValue, caption;
		char *nm = strdup(strName.c_str()), *vl = NULL;

		cfg.getAttribute(&nm, &vl, true);
		if (vl)
		{
			attributeValue = vl;
			free(vl);
		}
		free(nm);
		caption = displayCaption(infoDisp,
			attributeValue, "changing directory to: ", cmd.c_str());
		infoDisp.dsp(ezoi::OILVL_INFO, caption.c_str());

#ifdef WIN32
		const char *strOriginal = cmd.c_str();
		TCHAR *strConverted = char2TCHAR_begin(strOriginal);

		if (strConverted != NULL)
		{
			result = SetCurrentDirectory(strConverted);
			if (result != 0)
				result = 0;
			else
				result = -1;
			char2TCHAR_end(strOriginal, strConverted);
		}
#else
		result = chdir(cmd.c_str());
#endif
		if (result < 0)
		{
			std::ostringstream failureInfo;

			failureInfo << "failure (" << result
				<< ") in " << caption;
			infoDisp.dsp(ezoi::OILVL_WARN, failureInfo.str().c_str());
		}
	}
	else
	{
		infoDisp.dsp((ezoi::OILVL_WARN),
			"no directory to change to in an element");
		result = 1;
	}
	return result;
}

int ezcmd::runSetExec(ezoi &infoDisp, char *name,
	char *value, bool verbose)
{
	std::ostringstream info;
	int ret = 0;

	info << (name ? name : "") << "=" << (value ? value : "");
	if ((ret =
#ifdef WIN32
		_putenv
#else
		putenv
#endif
		(const_cast<char*>(info.str().c_str()))) < 0)
	{
		infoDisp.dsp(ezoi::OILVL_WARN,
			(std::string("failed to set: " + info.str())).c_str());
	}
	else if (verbose)
		infoDisp.dsp(ezoi::OILVL_INFO,
		(std::string("set: " + info.str())).c_str());

	return ret;
}

int ezcmd::runSet(ezcfg &cfg, ezoi &infoDisp)
{
	int result = 0;
	bool processed = false, verbose = true;
	std::string caption;
	char *nm = strdup(strName.c_str()), *vl = NULL,
		*text = NULL;

	cfg.getText(&text);
	cfg.getAttribute(&nm, &vl, true);
	if (nm && strlen(nm) && vl)
	{
		verbose = false;
		if (!strlen(text))
			infoDisp.dsp(ezoi::OILVL_INFO, vl);
		else
			caption = vl;
		free(nm);
		nm = NULL;
	}

	cfg.getAttribute(&nm, &vl, true);
	skipKeyParams(cfg, &nm, &vl);
	while (nm && strlen(nm))
	{	/* value in the file */
		processed = true;
		if (runSetExec(infoDisp, nm, vl, verbose) < 0)
			result = -1;
		free(nm);
		nm = NULL;
		cfg.getAttribute(&nm, &vl);
		skipKeyParams(cfg, &nm, &vl);
	}
	free(nm);
	free(vl);
	nm = vl = NULL;
	if (strlen(text))
	{	/* input the value */
		char *textValue = NULL;

		infoDisp.inpt(displayCaption(infoDisp, caption,
			"input the value of ", text).c_str(), &textValue);
		processed = true;
		if (runSetExec(infoDisp, text, textValue, verbose) < 0)
			result = -1;
	}

	if (!processed)
	{
		infoDisp.dsp((ezoi::OILVL_WARN),
			"no variables to set in an element");
		result = 1;
	}
	free(text);

	return result;
}

int ezcmd::runGoto(ezcfg &cfg, ezoi &infoDisp)
{
	int result = -1;
	bool processed = false, userInput = false;
	std::string caption;
	char *nm = strdup(strName.c_str()), *vl = NULL,
		*text = NULL, *place = NULL;

	cfg.getAttribute(&nm, &vl, true);
	if (nm && strlen(nm) && vl)
	{
		caption = vl;
		free(nm);
		nm = NULL;
	}

	cfg.getText(&text);
	cfg.getAttribute(&nm, &vl, true);
	skipKeyParams(cfg, &nm, &vl);
	if (nm && strlen(nm))
	{	/* need user to input */
		userInput = true;
		infoDisp.inpt(displayCaption(infoDisp, caption,
			"where to go?", "").c_str(), &place);
		if (0 != strcmp(nm, place))
		{
			free(nm);
			nm = place;
			cfg.getAttribute(&nm, &vl);
		}
		if (nm && strlen(nm))
			/* strictly paired */
			place = vl;
		else
		{
			if (strlen(text))
				place = text;
		}
		free(nm);
		nm = NULL;
	}
	else if (strlen(text))
		place = text;

	if (place && strlen(place))
	{
		if (!cfg.getBookmark(place))
		{
			std::ostringstream info;

			info << "label " << (place ? place : "") << " not defined";
			infoDisp.dsp(ezoi::OILVL_WARN, info.str().c_str());
		}
		else
		{
			if (!userInput)
				infoDisp.dsp(ezoi::OILVL_INFO,
					displayCaption(infoDisp, caption,
					"jumped to", place).c_str());
			result = 0;
		}
	}
	else
		infoDisp.dsp(ezoi::OILVL_WARN,
			"no defined label to jump to");
	free(place);

	return result;
}

std::string ezcmd::replaceText(ezcfg &cfg, ezoi &infoDisp)
{
	bool firstRun = true;
	char *nm = strdup(strInput.c_str()), *vl = NULL, *tx = NULL,
		*answer = NULL;
	std::string text;
	std::ostringstream info;
	std::string::size_type found, lastPlace;

	cfg.getText(&tx);
	if (tx)
		text = tx;
	if (text.length() > 0)
	{
		while(1)
		{
			cfg.getAttribute(&nm, &vl, firstRun);
			if (!nm || !strlen(nm))
				break;
			firstRun = false;
			info << "what to replace " << (vl ? vl : "");
			lastPlace = 0;
			while ((found = text.find(vl, lastPlace))
				!= std::string::npos)
			{
				if (!answer || !strlen(answer))
					infoDisp.inpt(info.str().c_str(), &answer);
				text.replace(found, vl ? strlen(vl) : 0, answer);
				lastPlace = found + answer ? strlen(answer) : 0;
			}
			free(answer);
			answer = NULL;
			info.flush();
		}
	}
	free(nm);
	free(vl);
	free(tx);

	return text;
}

ezcmd::eflow ezcmd::chkPermission(ezcfg &cfg, ezoi &infoDisp,
	std::bitset<flagQuantity> &flags)
{
	ezcmd::eflow ret = FLOW_GOON;
	char *nm = strdup(strPrompt.c_str()), *vl = NULL;

	cfg.getAttribute(&nm, &vl, true);
	if (nm || flags[flagPrompt])
	{
		/* construct prompt */
		std::ostringstream prompt;

		prompt << "continue to ";
		free(nm);
		free(vl);
		vl = NULL;
		nm = strdup(strName.c_str());
		cfg.getAttribute(&nm, &vl, true);
		if ((nm || flags[flagPrompt]) && vl)
		{	/* display user defined caption */
			prompt << vl;
		}
		else
		{
			char *value = NULL;

			cfg.getName(&value);
			/* process different element types */
			if (value && (strCmd == value))
			{
				cfg.getText(&value);
				if (!value || !strlen(value))
					prompt << "execute nothing";
				else
					prompt << "execute " << value;
			}
			else if (value && (strChd == value))
			{
				cfg.getText(&value);
				if (!value || !strlen(value))
					prompt << "change directory to nowhere";
				else
					prompt << "change directory to " << value;
			}
			else if (value && (strSet == value))
			{
				prompt << "set variable(s)";
				free(nm);
				nm = NULL;
				cfg.getAttribute(&nm, &vl, true);
				while (nm && strlen(nm))
				{
					if ((nm != strName)
						&& (nm != strPrompt)
						&& (nm != strStrict))
						prompt << nm;
					free(nm);
					nm = NULL;
					cfg.getAttribute(&nm, &vl);
				}
			}
			else if (value && (strGoto == value))
				prompt << "labeled flow control";
			else
				prompt << "act undefined";
			free(value);
		}
		prompt << "?(Skip/No/...)";

		char answer;

		infoDisp.inpt(prompt.str().c_str(), answer);
		if (('n' == answer) || ('N' == answer))
		{
			infoDisp.dsp(ezoi::OILVL_INFO, "now exits");
			ret = FLOW_STOP;
		}
		else
		if (('s' == answer) || ('S' == answer))
		{
			infoDisp.dsp(ezoi::OILVL_INFO, "action skipped");
			ret = FLOW_SKIP;
		}
	}
	free(nm);
	free(vl);
	return ret;
}

bool ezcmd::chkAbortion(int result, ezcfg &cfg,
	std::bitset<flagQuantity> &flags)

{
	if ((flags[flagIgnore]) || (0 == result))
		return false;
	if ((result < 0) || (flags[flagStrict]))
		return true;

	char *nm = strdup(strStrict.c_str()), *vl = NULL;
	bool ret = false;

	cfg.getAttribute(&nm, &vl, true);
	if (nm && strlen(nm))
		ret = true;
	free(nm);
	free(vl);
	return ret;
}

/*
return value:	callbacks' worst return value (smallest), or,
		1: user chose to stop; 0: succeeded;
		-1: procedure stopped because of error(s);
		-2: nothing to process at all
*/
int ezcmd::runExec(ezcfg &cfg,
	std::vector<std::pair<std::string, runCmdFunc> > &criteria,
	int requiredLevel, ezoi &infoDisp, std::bitset<flagQuantity> &flags)
{
	erttp next;
	char *elementName = NULL;
	int currentLevel = 0;
	int result = -2, lastResult = -2;
	bool visited = false;
	enum {
		RUNEXEC_PROCESSED_NOT,
		RUNEXEC_PROCESSED_YES,
		RUNEXEC_PROCESSED_NO_MORE,
		RUNEXEC_PROCESSED_NO_ADVANCE
	} processed = RUNEXEC_PROCESSED_NOT;

	while (true)
	{
		/* batch procedure */
		if (RUNEXEC_PROCESSED_NO_MORE == processed)
			break;
		if (processed != RUNEXEC_PROCESSED_NO_ADVANCE)
		{
			next = cfg.next();
			if (RTTP_CRER == next)
				break;
			if (!cfg.isSolid())
				continue;
			switch (next)
			{
				case RTTP_NOER:
					currentLevel++;
					if ((1 == currentLevel) && (0 == requiredLevel))
						infoDisp.dsp(ezoi::OILVL_INFO,
							"element(s) reside(s) in the wrong level of XML");
					break;
				case RTTP_MNER:
					currentLevel--;
					break;
				default:
					break;
			}
			if ((requiredLevel > 0) && (next != RTTP_NOER) && !visited)
			{	//if function is to be invoked, the node should be the nearest child
				lastResult = -1;
				infoDisp.dsp(ezoi::OILVL_FATAL,
					"no element found in the required position of XML");
				break;
			}
			visited = true;

			if (currentLevel > requiredLevel)
				continue;
			if (currentLevel < requiredLevel)
			{
				if (requiredLevel < 0)
					continue;
				lastResult = -1;
				infoDisp.dsp(ezoi::OILVL_FATAL,
					"no element found in valid range of XML");
				break;
			}
		}
		else
		{
			processed = RUNEXEC_PROCESSED_NOT;
			if (!cfg.isSolid())
				continue;
		}

		switch (chkPermission(cfg, infoDisp, flags))
		{
			case FLOW_STOP:
				lastResult = 1;
				processed = RUNEXEC_PROCESSED_NO_MORE;
				continue;
			case FLOW_SKIP:
				processed = RUNEXEC_PROCESSED_YES;
				continue;
			case FLOW_GOON:
				processed = RUNEXEC_PROCESSED_NOT;
		}

		/* single procedure */
		cfg.getName(&elementName);

		std::vector<std::pair<std::string, runCmdFunc> >::const_iterator it;

		for (it = criteria.begin(); it != criteria.end(); it++)
		{	/* find how to handle this procedure */
			if (elementName == it->first)
			{
				if (it->second)
				{	/* carry out the procedure */
					result = it->second(cfg, infoDisp);
					if ((lastResult > result) || ((result > 0)
						&& (0 == lastResult)))
						lastResult = result;

					if (chkAbortion(result, cfg, flags))
					{
						infoDisp.dsp(ezoi::OILVL_FATAL,
							"now aborts");
						processed = RUNEXEC_PROCESSED_NO_MORE;
					}
					else if (strGoto == elementName)
						processed = RUNEXEC_PROCESSED_NO_ADVANCE;
					else
						processed = RUNEXEC_PROCESSED_YES;
				}
				else
				{
					lastResult = 0;
					processed = RUNEXEC_PROCESSED_NO_MORE;
				}
				break;
			}
		}
		if (RUNEXEC_PROCESSED_NOT == processed)
			infoDisp.dsp(ezoi::OILVL_INFO,
				"one unrecognizable element resides in XML");
	}

	if (!visited)
		infoDisp.dsp(ezoi::OILVL_FATAL,
			"no useful element found in XML");
	else
	{
		if (RTTP_CRER == next)
			infoDisp.dsp(ezoi::OILVL_INFO,
				"batch ends");
	}
	return lastResult;
}

int ezcmd::run(char *argFileConfig, ezoi &infoDisp,
	std::bitset<flagQuantity> &flags)
{
	int runResult = -1;
	ezcfg cfg(argFileConfig);
	std::vector<std::pair<std::string, runCmdFunc> > vec;

	if (!cfg.isLoaded())
	{
		infoDisp.dsp(ezoi::OILVL_FATAL, "XML not loaded");
	}
	else
	{	/* go over all the labels */
		erttp next;
		char *nm = NULL, *vl = NULL;

		while ((next = cfg.next()) != RTTP_CRER)
		{
			if (!cfg.isSolid())
				continue;
			switch (next)
			{
				case RTTP_MNER:
				case RTTP_NOER:
				case RTTP_RGHT:
					nm = strdup(strLabel.c_str());
					cfg.getAttribute(&nm, &vl, true);
					if (nm && strlen(nm))
						cfg.setBookmark(vl);
			}
		}
		free(nm);
		free(vl);
		nm = vl = NULL;

		/* now process the batch */
		cfg.home();
		vec.push_back(std::make_pair<std::string,
			runCmdFunc>(strDoc, NULL));
		if (0 == (runResult = runExec(cfg, vec, 0, infoDisp, flags)))
		{
			vec.clear();
			vec.push_back(std::make_pair<std::string,
				runCmdFunc>(strCmd, runCmd));
			vec.push_back(std::make_pair<std::string,
				runCmdFunc>(strChd, runChd));
			vec.push_back(std::make_pair<std::string,
				runCmdFunc>(strSet, runSet));
			vec.push_back(std::make_pair<std::string,
				runCmdFunc>(strGoto, runGoto));
			runResult = runExec(cfg, vec, 1, infoDisp, flags);
		}
		else
		{
			if (-2 == runResult)
				infoDisp.dsp(ezoi::OILVL_FATAL,
				"no useful XML content");
			runResult = -1;
		}
	}

	return runResult;
}

}	/*	namespace ezproject	*/


static void hlp()
{
	std::cout << "EZ Command 1.3, by Allen, http://ezproject.sourceforge.net"
		<< std::endl
		<< "\tplease refer to README for license and usage information"
		<< std::endl
		<< "ezcmd {xml file name} [-i] [-s] [-p] [-o {ez command output file name}]"
		<< std::endl
		<< "i: ignore failures\ts: strict failure detection"
		<< std::endl
		<< "p: prompt before actions\to: output to file" << std::endl;
}

int main(int argc, char *argv[])
{
	/* command line parse */
	char *argFileConfig = NULL;
	char *argFileOutput = NULL;
	std::bitset<ezproject::ezcmd::flagQuantity> flags;

	if (argc > 1)
	{
		const char * const NOTRECOGNIZED = " not recognized in command line!";
#define CHECKREDUNDANT(ARGV, POS) if (ARGV[POS][2] != '\0') \
	std::cout << "'" << ARGV[POS] << "' taken as option '" \
	<< ARGV[POS][1] << "' in command line!" << std::endl;

		argFileConfig = argv[1];
		for (int i = 2; i < argc; i++)
		{
			if ('-' == *argv[i])
			{
				switch(argv[i][1])
				{
					case 'i':
						flags.set(ezproject::ezcmd::flagIgnore);
						CHECKREDUNDANT(argv, i);
						break;
					case 's':
						flags.set(ezproject::ezcmd::flagStrict);
						CHECKREDUNDANT(argv, i);
						break;
					case 'o':
						CHECKREDUNDANT(argv, i);
						if (++i < argc)
							argFileOutput = argv[i];
						else
							std::cout << "option '"
								<< argv[i - 1][1]
								<< "' lacks a parameter after it in command line!"
								<< std::endl;
						break;
					case 'p':
						flags.set(ezproject::ezcmd::flagPrompt);
						CHECKREDUNDANT(argv, i);
						break;
					default:
						std::cout << "option '"
							<< argv[i][1] << "'"
							<< NOTRECOGNIZED
							<< std::endl;
				}
			}
			else
				std::cout << "parameter '" << argv[i]
					<< "'" << NOTRECOGNIZED << std::endl;
		}
	}
	if (NULL == argFileConfig)
	{
		hlp();
		return 1;
	}

	ezproject::ezoi infoDisp;
	if (argFileOutput && (strlen(argFileOutput) > 0))
		if (infoDisp.file(ezproject::ezoi::FLTP_OUTPUT
			| ezproject::ezoi::FLTP_ERROR, argFileOutput)
			!= ezproject::RTTP_RGHT)
		{
			std::cout << "failed to write to " << argFileOutput
				<< std::endl;
		}

	std::string modName;
	modName = '{';	//start of header
	modName	+= "EZ Command";
	modName += '}';	//start of text
	infoDisp.mod(modName.c_str());

	ezproject::ezcmd cmd;
	int runResult = cmd.run(argFileConfig, infoDisp, flags);

	return runResult;
}

